// Ao40Fec.h: interface for the CAo40Fec class.
//
////////////////////////////////////////////////////////////////////
//				Copyright 1997 Phil Karn, KA9Q
////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_AO40FEC_H__2BB5F51B_2CB3_41FF_9538_F9DB2A6508D9__INCLUDED_)
#define AFX_AO40FEC_H__2BB5F51B_2CB3_41FF_9538_F9DB2A6508D9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define BLOCKSIZE  256          /* Data bytes per frame */
#define SYMRATE    400          /* Symbol rate, Hz (must be divisible into SAMPRATE) */
#define PNPOLY     0x48         /* Sync vector PN polynomial */
#define SCRAMBLER_POLY 0x95     /* Scrambler polynomial */
#define ICRATE     2            /* Inverse of convolutional encoder rate */
#define CK         7            /* Convolutional encoder constraint length */
#define ROWS       80           /* Block interleaver rows */
#define COLUMNS    65           /* Block interleaver columns */
#define RSBLOCKS   2            /* Reed-Solomon blocks per frame */
#define SCRAMBLE   1            /* 1 = enable scrambler, 0 = disable */

/* Derived and fixed constants - don't change */
#define SYMPBLOCK (ROWS*COLUMNS)      /* Encoded symbols per block */
#define E32 (1LL<<32)
#define PI2RAD (1<<30)         /* Pi/2 radians (90 deg), as phase increment */

#define NN 255    /* Total bytes per unshortened RS code block */
#define KK 223    /* Data bytes per unshortened RS code block */
#define RSDATA    (BLOCKSIZE/RSBLOCKS)    /* Data bytes per shortened RS code block */
#define RSPAD     (KK-RSDATA)             /* Padding bytes per RS code block */
#define RSTOT     (NN-KK+RSDATA)          /* Total bytes (data + parity) per shortened RS code block */

class CAo40Fec  
{
public:
	CAo40Fec();
	virtual ~CAo40Fec();
	// call this with 256 byte data array in pData
	// returns 5200 encoded symbols in pOutput
	void EncodeFrame(int* pOutput, unsigned char* pData);

private:
	//functions
	void Init();
	int InitRS(int symsize,int gfpoly,int fcr,int prim, int nroots,int pad);
	void EncodeRS( unsigned char *data, unsigned char *bb, int pad);
	int Modnn(int x);
	//variables
	int m_SyncVector[COLUMNS];
	unsigned char m_Scrambler[BLOCKSIZE+RSBLOCKS*(NN-KK)];
	//inline functions
	inline int update(int x)
	{
		if(x == SYMPBLOCK-1)
			return -1; /* Was last symbol, can't update */
		x += ROWS;
		if(x >= SYMPBLOCK)
			x -= (SYMPBLOCK-1);
		return x;
	}
	inline int parityb(unsigned char x)
	{
		__asm
		{
			test x,0xFF
			setpo x
		}
		return (int)x;
	}
	inline int parity(int x)
	{	//Fold down to one byte
		x ^= (x >> 16);
		x ^= (x >> 8);
		return parityb(x);
	}
	inline int mod255(int x)
	{
		while (x >= 255)
		{
			x -= 255;
			x = (x >> 8) + (x & 255);
		}
		return x;
	}
	struct rs	// Reed-Solomon codec control block
	{
		int mm;              /* Bits per symbol */
		int nn;              /* Symbols per block (= (1<<mm)-1) */
		unsigned char *alpha_to;     /* log lookup table */
		unsigned char *index_of;     /* Antilog lookup table */
		unsigned char *genpoly;      /* Generator polynomial */
		int nroots;     /* Number of generator roots = number of parity symbols */
		int fcr;        /* First consecutive root, index form */
		int prim;       /* Primitive element, index form */
		int iprim;      /* prim-th root of 1, index form */
		int pad;        /* Padding bytes in shortened block */
	}m_RS;
};

#endif // !defined(AFX_AO40FEC_H__2BB5F51B_2CB3_41FF_9538_F9DB2A6508D9__INCLUDED_)
