// BProgress.cpp : implementation file
//
//////////////////////////////////////////////////////////////////////
// Copyright 2000.    Moe Wheatley AE4JY  <ae4jy@qsl.net>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BProgressCtrl.h"
#include "AO40RcvDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MIN_POS 0						//control range
#define MAX_POS 45

#define SYNC_COLOR RGB(255,255,0)		// yellow sync search
#define NOSIG_COLOR RGB(128,128,128)	// grey looking for signal
#define READ_COLOR RGB(0,255,255)		// cyan reading block
#define OK_COLOR RGB(0,255,0)			// green crc ok m_Color
#define BAD_COLOR RGB(255,0,0)			// Red crc bad m_Color
#define TEXT_COLOR RGB(0,0,0)			// prog text color
#define TEXTOK_COLOR RGB(0,0 ,255)		// crcok text color
#define READTEXT_COLOR RGB(0,0,255)		// blue text reading block
#define TEXT2_COLOR RGB(64,64,128)   	// prog text color under bar

/////////////////////////////////////////////////////////////////////////////
// CBProgressCtrl

CBProgressCtrl::CBProgressCtrl()
{
    RegisterWindowClass();
	m_nValue = 0;
	m_BkColor = GetSysColor(COLOR_MENU);
	m_TxtColor = TEXT_COLOR;
	m_BarColor = NOSIG_COLOR;
	m_LEDColor = NOSIG_COLOR;
	m_ColorDelay = FALSE;
}

CBProgressCtrl::~CBProgressCtrl()
{
}


// Register the window class if it has not already been registered.
BOOL CBProgressCtrl::RegisterWindowClass()
{
    WNDCLASS wndcls;
    HINSTANCE hInst = AfxGetInstanceHandle();

    if (!(::GetClassInfo(hInst, CBPROGRESS_CLASSNAME, &wndcls)))
    {
        // otherwise we need to register a new class
        wndcls.style            = CS_HREDRAW | CS_VREDRAW;
        wndcls.lpfnWndProc      = ::DefWindowProc;
        wndcls.cbClsExtra       = wndcls.cbWndExtra = 0;
        wndcls.hInstance        = hInst;
        wndcls.hIcon            = NULL;
        wndcls.hCursor          = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
        wndcls.hbrBackground    = NULL;	//am drawing own background color
        wndcls.lpszMenuName     = NULL;
        wndcls.lpszClassName    = CBPROGRESS_CLASSNAME;

        if (!AfxRegisterClass(&wndcls))
        {
            AfxThrowResourceException();
            return FALSE;
        }
    }

    return TRUE;
}

BEGIN_MESSAGE_MAP(CBProgressCtrl, CWnd)
	//{{AFX_MSG_MAP(CBProgressCtrl)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////
//  Creates the window
///////////////////////////////////////////////////////////////////////////
BOOL CBProgressCtrl::Create( DWORD dwStyle, const RECT& rect,
							CWnd* pParentWnd, UINT nID)
{
	return CWnd::Create(CBPROGRESS_CLASSNAME, _T(""), dwStyle, rect, pParentWnd, nID);

}

void CBProgressCtrl::PreSubclassWindow() 
{
	ModifyStyleEx (0, WS_EX_CLIENTEDGE|WS_EX_NOPARENTNOTIFY);
	ModifyStyle (0, WS_BORDER|WS_CHILD|WS_VISIBLE, SWP_DRAWFRAME);
//	ModifyStyle (0, WS_DLGFRAME|WS_CHILD|WS_VISIBLE, SWP_DRAWFRAME);
	
	CWnd::PreSubclassWindow();
}
///////////////////////////////////////////////////////////////////////////
//  Set progress value
///////////////////////////////////////////////////////////////////////////
void CBProgressCtrl::SetValue(INT nValue)
{
	if(nValue< MIN_POS)
		nValue = MIN_POS;
	if(nValue > MAX_POS)
		nValue = MAX_POS;
	m_nValue = nValue;
	InvalidateCntrl();
}

void CBProgressCtrl::InvalidateCntrl()
{
	CRect r;
	GetClientRect(r);
	InvalidateRect(r);
}

void CBProgressCtrl::SetProgressState(INT state, CString Str)
{
	switch( state)
	{
		case PROG_STATE_OFF:
			m_BarColor = NOSIG_COLOR;
			m_TxtColor = TEXT_COLOR;
			m_LEDColor = NOSIG_COLOR;
			m_ColorDelay = FALSE;
			break;
		case PROG_STATE_NOSIG:
			m_BarColor = NOSIG_COLOR;
			m_TxtColor = TEXT_COLOR;
			m_ColorDelay = FALSE;
			m_LEDColor = NOSIG_COLOR;
			break;
		case PROG_STATE_SYNC:
			if( !m_ColorDelay)
			{
				m_BarColor = SYNC_COLOR;
			}
			break;
		case PROG_STATE_FECSYNC:
			m_BarColor = READ_COLOR;
			m_LEDColor = NOSIG_COLOR;
			break;
		case PROG_STATE_READ:
			m_ColorDelay = TRUE;
			m_BarColor = READ_COLOR;
			m_LEDColor = NOSIG_COLOR;
			m_TxtColor = READTEXT_COLOR;
			break;
		case PROG_STATE_GOOD:
			m_BarColor = OK_COLOR;
			m_LEDColor = OK_COLOR;
			m_TxtColor = TEXTOK_COLOR;
			break;
		case PROG_STATE_BAD:
			m_BarColor = BAD_COLOR;
			m_LEDColor = BAD_COLOR;
			m_TxtColor = BAD_COLOR;
			break;
	}
	this->SetWindowText( Str);
	InvalidateCntrl();
}

/////////////////////////////////////////////////////////////////////////////
// CBProgressCtrl message handlers
//
//       Called to draw the control
/////////////////////////////////////////////////////////////////////////////
void CBProgressCtrl::OnPaint() 
{
CDC MemDC;
CPaintDC dc(this); // device context for painting
CRect rClient;
CRect rLED;
CRect rLEDBlock;
CRect rProg;
//create various rectangles for all the viewable thingys
	GetClientRect( rClient );	
	rLEDBlock.SetRect(rClient.right-rClient.Height(),rClient.top,rClient.right,rClient.bottom);
	rProg.SetRect( rClient.left,rClient.top,rClient.right-rClient.Height(),rClient.bottom);
	rLED.SetRect(rLEDBlock.left+3,rLEDBlock.top+3,rLEDBlock.right-3,rLEDBlock.bottom-3);

	if( m_bmImage.m_hObject != NULL )	// delete old bitmap object
		m_bmImage.DeleteObject();
//create a new bitmap with current screen DC settings
	m_bmImage.CreateCompatibleBitmap(&dc,rClient.Width(),rClient.Height() );
//create memory DC pointer to the image bitmap
	MemDC.CreateCompatibleDC( &dc);
	CBitmap* pbmOld = MemDC.SelectObject( &m_bmImage );

//Create 2 rectangles for the progress bar depending upon m_nValue. 	
	CRect rLeft = rProg;
	CRect rRight = rProg;
	rLeft.right = (rProg.Width()*m_nValue)/MAX_POS;
	rRight.left = rLeft.right;

//Create two corresponding regions for use in clipping the text areas
	CRgn rgnLeft;
	CRgn rgnRight;
	CRgn rgnLED;
//	rgnLED.CreateEllipticRgnIndirect(&rLED);
	rgnLED.CreateRectRgnIndirect(&rLED);
	rgnLeft.CreateRectRgnIndirect(&rLeft);
	rgnRight.CreateRectRgnIndirect(&rRight);

//Fill Progress bar with bar color
	MemDC.FillSolidRect(&rLeft,m_BarColor);
//Fill non Progress bar with background color
	MemDC.FillSolidRect(&rRight,m_BkColor);
//Fill round LED area with color and the background black
	MemDC.FillSolidRect(&rLEDBlock,RGB(0,0,0));
	MemDC.SelectClipRgn(&rgnLED);
	MemDC.FillSolidRect(&rLED,m_LEDColor);

//Get status bar text to place in the window
	CString strCaption;
	GetWindowText(strCaption);

//Draw with backgrnd color the part under the progress bar
	MemDC.SelectClipRgn(&rgnLeft);
	MemDC.SetBkMode(TRANSPARENT);
	MemDC.SetTextColor( TEXT2_COLOR );
	MemDC.DrawText(strCaption, rProg,DT_VCENTER|DT_SINGLELINE|DT_CENTER);

//Draw with normal text color the part NOT under the progress bar
	MemDC.SelectClipRgn(&rgnRight);
	MemDC.SetTextColor( m_TxtColor );
	MemDC.DrawText(strCaption, rProg,DT_VCENTER|DT_SINGLELINE|DT_CENTER);
// Copy bitmap into screen context
	dc.BitBlt(	rClient.left, rClient.top,		//dest left,top
				rClient.right,	rClient.bottom,	//dest width, Height
				&MemDC,							//src dc
				0,0,							//src left, top
				SRCCOPY);
//deselect bitmap
	MemDC.SelectObject( pbmOld );	// from DC
	rgnLED.DeleteObject();
	rgnLeft.DeleteObject();
	rgnRight.DeleteObject();

}
