// InSelect.cpp : implementation file
//
//////////////////////////////////////////////////////////////////////
// Copyright 2000.    Moe Wheatley AE4JY  <ae4jy@qsl.net>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AO40Rcv.h"
#include "InSelect.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define SOUND_GRP 0
#define WAVE_GRP 1
#define RAW_GRP 2

/////////////////////////////////////////////////////////////////////////////
// CInSelect dialog


CInSelect::CInSelect(CWnd* pParent /*=NULL*/)
	: CDialog(CInSelect::IDD, pParent)
{
	//{{AFX_DATA_INIT(CInSelect)
	m_InGroup = 0;
	m_InRawName = _T("");
	m_InWaveName = _T("");
	m_WaveStartOffset = 0;
	//}}AFX_DATA_INIT
	m_InRawPath = _T("");
	m_InWavePath = _T("");
	m_MaxBlocks = 1;
}


void CInSelect::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CInSelect)
	DDX_Control(pDX, IDC_FILEDATA, m_FileDataCtrl);
	DDX_Control(pDX, IDC_SCARDCOMBO, m_SoundCardCtrl);
	DDX_Control(pDX, IDC_BLOCKTXT, m_BlkTxtCtrl);
	DDX_Radio(pDX, IDC_INGROUPSOUND, m_InGroup);
	DDX_Text(pDX, IDC_INRAWNAME, m_InRawName);
	DDX_Text(pDX, IDC_INWAVENAME, m_InWaveName);
	DDX_Text(pDX, IDC_WAVEOFFSET, m_WaveStartOffset);
	DDV_MinMaxUInt(pDX, m_WaveStartOffset, 0, 10000);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CInSelect, CDialog)
	//{{AFX_MSG_MAP(CInSelect)
	ON_BN_CLICKED(IDC_SELRAW, OnSelraw)
	ON_BN_CLICKED(IDC_SELWAVE, OnSelwave)
	ON_BN_CLICKED(IDC_INGROUPWAVE, OnIngroupwave)
	ON_BN_CLICKED(IDC_INGROUPSOUND, OnIngroupsound)
	ON_BN_CLICKED(IDC_INGROUPRAW, OnIngroupraw)
	ON_CBN_KILLFOCUS(IDC_SCARDCOMBO, OnKillfocusScardcombo)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CInSelect message handlers
BOOL CInSelect::OnInitDialog() 
{
CString str;
UINT num = 0;
WAVEINCAPS wic;
	CDialog::OnInitDialog();
	m_SoundCardCtrl.AddString( "Use Default SoundCard" );
	num = waveInGetNumDevs();
	for(UINT i=0; i<num; i++)
	{
		waveInGetDevCaps(  i, &wic, sizeof(WAVEINCAPS) );
		m_SoundCardCtrl.AddString( wic.szPname );
	}
	m_SoundCardCtrl.SetCurSel( m_SoundCard+1);
	EnableControls(m_InGroup);
	if(m_InGroup == RAW_GRP)
	{
		FindMaxBlocks();
	}
	DisplayFileData();
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CInSelect::OnSelraw() 
{
//  handler code here for getting raw data Input filename
CString sFilter = "Good CRC Telemetry Files (*.tlm)|*.tlm|Raw Telemetry Files (*.raw)|*.raw||";
CFileDialog Dlgfile( TRUE, NULL, m_InRawPath, 
			OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT,
			(LPCTSTR)sFilter, this );
	if( Dlgfile.DoModal() == IDOK)
	{
		m_InRawName = Dlgfile.GetFileName();
		m_InRawPath = Dlgfile.GetPathName();
		FindMaxBlocks();
	}
	UpdateData(FALSE);		//put new data in control
}

void CInSelect::OnSelwave() 
{
//  handler code here for getting wave Input filename
CString sFilter = "Wave Files (*.wav)|*.wav||";
CFileDialog Dlgfile( TRUE, "wav", m_InWavePath, 
			OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT,
			(LPCTSTR)sFilter, this );
	if( Dlgfile.DoModal() == IDOK)
	{
		m_InWaveName = Dlgfile.GetFileName();
		m_InWavePath = Dlgfile.GetPathName();
		DisplayFileData();
	}
	UpdateData(FALSE);		//put new data in control
}

void CInSelect::OnIngroupwave() 
{
	UpdateData(TRUE);
	EnableControls(m_InGroup);
}

void CInSelect::OnIngroupsound() 
{
	UpdateData(TRUE);
	EnableControls(m_InGroup);
}

void CInSelect::OnIngroupraw() 
{
	UpdateData(TRUE);
	EnableControls(m_InGroup);
}

void CInSelect::EnableControls(INT group)
{
CWnd* pWnd;
	switch(m_InGroup)
	{
		case SOUND_GRP:
			pWnd = GetDlgItem( IDC_SELWAVE ); ASSERT_VALID(pWnd);
			pWnd->EnableWindow(FALSE);
			pWnd = GetDlgItem( IDC_SELRAW ); ASSERT_VALID(pWnd);
			pWnd->EnableWindow(FALSE);
			pWnd = GetDlgItem( IDC_INWAVENAME ); ASSERT_VALID(pWnd);
			pWnd->EnableWindow(FALSE);
			pWnd = GetDlgItem( IDC_INRAWNAME ); ASSERT_VALID(pWnd);
			pWnd->EnableWindow(FALSE);
			m_BlkTxtCtrl.EnableWindow(FALSE);
			m_SoundCardCtrl.EnableWindow(TRUE);
			break;
		case WAVE_GRP:
			pWnd = GetDlgItem( IDC_SELWAVE ); ASSERT_VALID(pWnd);
			pWnd->EnableWindow(TRUE);
			pWnd = GetDlgItem( IDC_SELRAW ); ASSERT_VALID(pWnd);
			pWnd->EnableWindow(FALSE);
			pWnd = GetDlgItem( IDC_INWAVENAME ); ASSERT_VALID(pWnd);
			pWnd->EnableWindow(TRUE);
			pWnd = GetDlgItem( IDC_INRAWNAME ); ASSERT_VALID(pWnd);
			pWnd->EnableWindow(FALSE);
			m_BlkTxtCtrl.EnableWindow(FALSE);
			m_SoundCardCtrl.EnableWindow(FALSE);
			break;
		case RAW_GRP:
			pWnd = GetDlgItem( IDC_SELWAVE ); ASSERT_VALID(pWnd);
			pWnd->EnableWindow(FALSE);
			pWnd = GetDlgItem( IDC_SELRAW ); ASSERT_VALID(pWnd);
			pWnd->EnableWindow(TRUE);
			pWnd = GetDlgItem( IDC_INWAVENAME ); ASSERT_VALID(pWnd);
			pWnd->EnableWindow(FALSE);
			pWnd = GetDlgItem( IDC_INRAWNAME ); ASSERT_VALID(pWnd);
			pWnd->EnableWindow(TRUE);
			m_BlkTxtCtrl.EnableWindow(TRUE);
			m_SoundCardCtrl.EnableWindow(FALSE);
			FindMaxBlocks();
			break;
	}
}



void CInSelect::OnKillfocusScardcombo() 
{
	m_SoundCard = m_SoundCardCtrl.GetCurSel() - 1;
}

void CInSelect::FindMaxBlocks(void)
{
CFile cFile;
	if( cFile.Open( m_InRawPath, CFile::modeRead) )
	{
		m_NumBlocksInFile = cFile.GetLength()/512;
		if(m_NumBlocksInFile < 1)
		{
			m_InRawName = _T("File too small");
			m_InRawPath = "";
		}
		else
		{
			CString str;
			str.Format("%d",m_NumBlocksInFile);
			m_BlkTxtCtrl.SetWindowText(str);
		}
	}
}

void CInSelect::DisplayFileData()
{
CString str = "";
	if( fnCheckWavePath((LPCTSTR)m_InWavePath,&m_FileSize) )
	{
		AfxMessageBox( IDS_WAVEERROR );
		m_InWaveName = _T(" Invalid wave file");
	}
	else
	{
		str.Format( "File is %d Seconds at 8000sps",m_FileSize/8000);
	}
	m_FileDataCtrl.SetWindowText( str);
}
