// PlotData.cpp: implementation of the CPlotData class.
//
//////////////////////////////////////////////////////////////////////
// Copyright 2000.    Moe Wheatley AE4JY  <ae4jy@qsl.net>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////

#include "stdafx.h"
#include "AO40Rcv.h"
#include "PlotData.h"
#include "AO40RcvView.h"
#include "BPSK400Externals.h"
#include "PlotTables.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define MAX_DISPFREQ 3000	//max range of the frequency display in Hz
#define MIN_DISPFREQ 100	//min range of the frequency display in Hz
#define MAX_FREQ (MAX_DISPFREQ*512/4000)	//max range of the frequency display in FFT units
#define MIN_FREQ (MIN_DISPFREQ*512/4000)	//min range of the frequency display in FFT units

const int Tri[20] =
{
	1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,0
};


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CPlotData::CPlotData()
{
	m_pDoc = NULL;

//allocate memory space
	m_pDispBuf = new INT[DISPBUF_SIZE];
	ASSERT( m_pDispBuf );
	m_LastOvrLoad = FALSE;
	m_BufToggle = FALSE;
//allocate FFT object
	m_pPenR = new CPen;		// create some colored pens for drawing
	m_pPenRW = new CPen;
	m_pPenG = new CPen;
	m_pPenY = new CPen;
	m_pPenW = new CPen;
	m_pPenC = new CPen;
	m_pPenB = new CPen;
	m_pPenR->CreatePen( PS_SOLID,0, RGB(255,0,0) );
	m_pPenRW->CreatePen( PS_SOLID,2, RGB(255,255,0) );
	m_pPenG->CreatePen( PS_SOLID,0, RGB(0,255,0) );
	m_pPenY->CreatePen( PS_DOT,0, RGB(255,255,0) );
	m_pPenW->CreatePen( PS_SOLID,0, RGB(128,128,128) );
	m_pPenC->CreatePen( PS_SOLID,0, RGB(0,255,255) );
	m_pPenB->CreatePen( PS_SOLID,0, RGB(0,0,255) );
	m_NeedToClrBufs = TRUE;
	m_RedrawScreen = FALSE;
	m_TicCounter = 0.0;
	m_Ave = 0.0;
	m_hViewWnd = NULL;
	m_rFreqPlot.bottom = 0;
	m_rFreqPlot.top = 0;
	m_rFreqPlot.left = 0;
	m_rFreqPlot.right = 0;
}

CPlotData::~CPlotData()
{
	m_pPenR->DeleteObject(); delete m_pPenR;
	m_pPenRW->DeleteObject(); delete m_pPenRW;
	m_pPenG->DeleteObject(); delete m_pPenG;
	m_pPenY->DeleteObject(); delete m_pPenY;
	m_pPenW->DeleteObject(); delete m_pPenW;
	m_pPenC->DeleteObject(); delete m_pPenC;
	m_pPenB->DeleteObject(); delete m_pPenB;
	if(m_pDispBuf)
	{
		delete m_pDispBuf;
		m_pDispBuf = NULL;
	}
// Delete two ping pong bitmap buffers for waterfall display
	if( m_bmWaterA.m_hObject != NULL )	// delete old bitmap object
		m_bmWaterA.DeleteObject();
	if( m_bmWaterB.m_hObject != NULL )	// delete old bitmap object
		m_bmWaterB.DeleteObject();
}


//////////////////////////////////////////////////////////////////////
//  Called from the Windows view class when the window has changed
//      and needs to update the screen coordinates.
//////////////////////////////////////////////////////////////////////
void CPlotData::OnDrawPlot(HWND hWnd, CRect crPlot, CAO40RcvDoc* pDoc)
{
CRect rScrn;
	m_pDoc = pDoc;
	m_hViewWnd = hWnd;
	if( crPlot != m_rFreqPlot)
	{
		m_NeedToClrBufs = TRUE;

		m_rFreqPlot = crPlot;
// Get pixel size of Freq plot area bitmap
		m_rFreqPlotsize.left = 0;
		m_rFreqPlotsize.right = m_rFreqPlot.Width();
		m_rFreqPlotsize.top = 0;
		m_rFreqPlotsize.bottom = m_rFreqPlot.Height();
// The translate table is used to map DispBuf[] index's into
//		Screen plot coordinates.
		for(INT i=MIN_FREQ; i<MAX_FREQ; i++)
		{
			m_TranslateTbl[i] = ( (i-MIN_FREQ)*m_rFreqPlotsize.right )
										/ (MAX_FREQ-MIN_FREQ );
		}
//create a waterfall ping pong bitmap with current screen DC settings
	CClientDC ClientDC(CWnd::FromHandle(m_hViewWnd) );
		if( m_bmWaterA.m_hObject != NULL )	// delete old bitmap object
			m_bmWaterA.DeleteObject();
		if( m_bmWaterB.m_hObject != NULL )	// delete old bitmap object
			m_bmWaterB.DeleteObject();
		m_bmWaterA.CreateCompatibleBitmap( &ClientDC, m_rFreqPlot.Width(), m_rFreqPlotsize.bottom );
		m_bmWaterB.CreateCompatibleBitmap( &ClientDC, m_rFreqPlot.Width(), m_rFreqPlotsize.bottom );
		fnSetFFTParams( 5, 255, 20 );
		double r = (double)m_rFreqPlotsize.bottom/16.0;
		int xc,yc;
		yc = m_rFreqPlotsize.bottom/16;
		xc = yc;
		double angle = 0.0;
		for(i=0; i<20; i++)
		{
			m_BitPosTable[i].x = xc + (long)(r*cos(angle));
			m_BitPosTable[i].y = yc + (long)(r*sin(angle));
			angle += (6.283185/20.0);
		}
	}
	if(m_RedrawScreen)
	{
		DrawPlot();
		m_RedrawScreen = FALSE;
	}
}


//////////////////////////////////////////////////////////////////////
//  Called to clear plot surface, draw cursor, and call specific plot
//         routine.
//////////////////////////////////////////////////////////////////////
void CPlotData::DrawPlot()
{
// erases plot surface
// plots data points
CDC MemDC;
CPen* pOldPen;
	if( m_hViewWnd == NULL )	//make sure OnDrwPlot has been called first
		return;
CClientDC ClientDC(CWnd::FromHandle(m_hViewWnd) );
	if( m_bmPlot.m_hObject != NULL )	// delete old bitmap objects
		m_bmPlot.DeleteObject();
//create a new bitmap with current screen DC settings
	m_bmPlot.CreateCompatibleBitmap( &ClientDC, m_rFreqPlot.Width(),
					 m_rFreqPlot.Height() );
//create memory DC pointer to the plot bitmap
	MemDC.CreateCompatibleDC( &ClientDC );

//==========Plot Frequency plot============
	CBitmap* pbmOld = MemDC.SelectObject( &m_bmPlot );
// set bitmap background to black using slightly faster ExtTextOut function
	MemDC.SetBkColor( RGB(0, 0, 0) );
	MemDC.ExtTextOut( 0, 0,ETO_OPAQUE, &m_rFreqPlotsize,NULL,0,NULL);

// Draw vertical yellow dotted plot cursor line
	pOldPen = MemDC.SelectObject(m_pPenY);
	MemDC.MoveTo( m_CurPos, 0);
	MemDC.LineTo( m_CurPos, m_rFreqPlotsize.bottom );
	MemDC.SelectObject(m_pPenG);
	PlotFFTWater( &MemDC );
	MemDC.SelectObject(m_pPenW);		//save original pen.
	for( INT i=10; i<=80; i+=10 )	//draw 10dB tic marks
	{
		MemDC.MoveTo( 0, (m_rFreqPlotsize.bottom*i)/(2*8*10) );
		MemDC.LineTo( m_rFreqPlotsize.right, (m_rFreqPlotsize.bottom*i)/(2*8*10) );
	}
	PlotOverlayVectData( &MemDC );
	// Copy memory bitmap into screen device context for flicker free update
	ClientDC.BitBlt( m_rFreqPlot.left, m_rFreqPlot.top,
					m_rFreqPlotsize.right,
					m_rFreqPlotsize.bottom, &MemDC,
					0,0,SRCCOPY);

/////////////////////////	
	MemDC.SelectObject(pOldPen);	//deselect pens and bitmap
	MemDC.SelectObject( pbmOld );	// from memory DC
}



//////////////////////////////////////////////////////////////////////
//		Perform FFT and plot waterfall
//////////////////////////////////////////////////////////////////////
void CPlotData::PlotFFTWater( CDC* pDC)
{
INT i,x,y,k1,k2,k3,k4;
COLORREF cr;
CDC MemDCA;		// memory DCs for the above bitmaps
CDC MemDCB;
BOOL ovrld;
BOOL oddsec = FALSE;
	
	if(m_TicCounter >= 5.0)
	{
		oddsec = TRUE;
		if(m_TicCounter >= 10.0)
			m_TicCounter -= 10.0;
	}
	m_TicCounter += .256;

	k4 = m_rFreqPlotsize.bottom;
//create memory DC pointers for the waterfall bitmap buffers
	MemDCA.CreateCompatibleDC( pDC );
	MemDCB.CreateCompatibleDC( pDC );
CBitmap* pbmOldA = MemDCA.SelectObject( &m_bmWaterA );
CBitmap* pbmOldB = MemDCB.SelectObject( &m_bmWaterB );
	if(m_NeedToClrBufs)
	{
		MemDCA.BitBlt( 0, 0, m_rFreqPlotsize.right,k4, NULL, 0,0,BLACKNESS);
		MemDCB.BitBlt( 0, 0, m_rFreqPlotsize.right,k4, NULL, 0,0,BLACKNESS);
		m_NeedToClrBufs = FALSE;
	}
	k3 = (1*k4)/2; //y starting point


	ovrld = fnGetFFTData( (long*)m_pDispBuf, MIN_FREQ, MAX_FREQ );

	pDC->MoveTo( 0, k3-(k3*m_pDispBuf[MIN_FREQ])/255);//move to first position
	for( x=MIN_FREQ; x<MAX_FREQ; x++ )
	{
		y = k3-(k3*m_pDispBuf[x])/255;
		pDC->LineTo( m_TranslateTbl[x], y );		//draw to the next pt.
	}
	k2 = (MAX_FREQ-MIN_FREQ+1);

	if(m_BufToggle) //use A buffer
	{
		// Copy lines from buf B to A shifted down 1 lines
		MemDCA.BitBlt( 0, 1, m_rFreqPlotsize.right,k4-k3-1, &MemDCB, 0,0,SRCCOPY);
		// Draw top two new lines int to buf A
		for( x=0; x<m_rFreqPlotsize.right; x++)
		{
			k1 = (x*k2)/m_rFreqPlotsize.right + MIN_FREQ;
			i = m_pDispBuf[k1];
			cr = ColorTbl[i];		//RGB( i, i, i);
			MemDCA.SetPixel( x , 0, cr );
		}
		if(oddsec)
			MemDCA.SetPixel( m_rFreqPlotsize.right-(INT)m_pDoc->m_DopplerShift/2-75 , 0, RGB(255,255,0) );
		else
			MemDCA.SetPixel( m_rFreqPlotsize.right-(INT)m_pDoc->m_DopplerShift/2-75 , 0, RGB(255,0,0) );
		// copy buf A into main plot bitmap
		pDC->BitBlt( 0, k3,	m_rFreqPlotsize.right,k4-k3, &MemDCA, 0,0,SRCCOPY);
	}
	else			// use B buffer
	{
		// Copy lines from buf A to B shifted down 1 lines
		MemDCB.BitBlt( 0, 1, m_rFreqPlotsize.right,k4-k3-1, &MemDCA, 0,0,SRCCOPY);
		// Draw top two new lines int to buf B
		for( x=1; x<m_rFreqPlotsize.right; x++)
		{
			k1 = (x*k2)/m_rFreqPlotsize.right + MIN_FREQ;
			i = m_pDispBuf[k1];
			cr = ColorTbl[i];		//RGB( i, i, i);
			MemDCB.SetPixel( x , 0, cr );
		}
		if(oddsec)
			MemDCB.SetPixel( m_rFreqPlotsize.right-(INT)m_pDoc->m_DopplerShift/2-75 , 0, RGB(255,255,0) );
		else
			MemDCB.SetPixel( m_rFreqPlotsize.right-(INT)m_pDoc->m_DopplerShift/2-75 , 0, RGB(255,0,0) );
		// copy buf B into main plot bitmap
		pDC->BitBlt( 0, k3,	m_rFreqPlotsize.right,k4-k3, &MemDCB, 0,0,SRCCOPY);
	}
	MemDCA.SelectObject( pbmOldA );
	MemDCB.SelectObject( pbmOldB );
	m_BufToggle = !m_BufToggle;
	if(ovrld && m_LastOvrLoad) //need two in a row
	{
		pDC->SetTextColor( RGB(255,0,0) );
		pDC->TextOut( m_rFreqPlotsize.right/4, m_rFreqPlotsize.bottom/8,
			_T("Reduce Audio Level"), 18 );
	}
	m_LastOvrLoad = ovrld;
}

//////////////////////////////////////////////////////////////////////
// Plot Baseband Data as vector plot on top of spectrum graph
//////////////////////////////////////////////////////////////////////
void CPlotData::PlotOverlayVectData( CDC* pDC )
{
INT x,xc,yc;
long VectArray[16];
CBrush* pOldBrush;
CBrush Brush;
	Brush.CreateSolidBrush( RGB(0,0,0) );
	yc = m_rFreqPlotsize.bottom/16;	// y center
	xc = yc;
	pOldBrush = pDC->SelectObject(&Brush);
	pDC->Ellipse( 0, 0, xc+xc, m_rFreqPlotsize.bottom/8);	//draw circle
	fnGetVectorData(VectArray);
	pDC->SelectObject(m_pPenC);
	for(x=0; x<16; x+=2)
	{
		pDC->MoveTo( xc,  yc );
		pDC->LineTo( xc+(yc*VectArray[x]/1000), 
							yc-((yc)*VectArray[x+1]/1000) );
	}
	pDC->SelectObject(m_pPenRW);
	x = fnGetBitPosition();
	pDC->MoveTo( m_BitPosTable[x] );
	pDC->LineTo( m_BitPosTable[ Tri[x] ]);
	pDC->LineTo( m_BitPosTable[x] );
	pDC->SelectObject( pOldBrush );

}

INT CPlotData::GetCursFreq(CPoint point)
{
INT range = 512*(m_rFreqPlot.right -m_rFreqPlot.left);
	return (MIN_DISPFREQ+((MAX_FREQ-MIN_FREQ)*4000*( point.x - m_rFreqPlot.left))/range);
}

void CPlotData::SetCursFreq(INT freq)
{
	m_CurPos = (512*(freq-MIN_DISPFREQ)*(m_rFreqPlot.right - m_rFreqPlot.left))/
					((MAX_FREQ-MIN_FREQ)*4000);
}

void CPlotData::RedrawScreen()
{
	m_RedrawScreen = TRUE;
}

void CPlotData::ClearBufs()
{
	m_NeedToClrBufs = TRUE;
}
